@@ -16,7 +16,7 @@
 
 package org.gradle.execution.taskgraph;
 
-import groovy.lang.Closure;
+import org.gradle.api.Action;
 import org.gradle.api.CircularReferenceException;
 import org.gradle.api.Task;
 import org.gradle.api.execution.TaskExecutionGraphListener;
@@ -24,7 +24,11 @@ import org.gradle.api.execution.TaskExecutionListener;
 import org.gradle.api.execution.internal.InternalTaskExecutionListener;
 import org.gradle.api.internal.TaskInternal;
 import org.gradle.api.internal.project.ProjectInternal;
-import org.gradle.api.internal.tasks.*;
+import org.gradle.api.internal.tasks.DefaultTaskDependency;
+import org.gradle.api.internal.tasks.DefaultTaskOutputs;
+import org.gradle.api.internal.tasks.TaskExecuter;
+import org.gradle.api.internal.tasks.TaskExecutionContext;
+import org.gradle.api.internal.tasks.TaskStateInternal;
 import org.gradle.api.specs.Spec;
 import org.gradle.api.tasks.TaskDependency;
 import org.gradle.api.tasks.TaskOutputs;
@@ -34,22 +38,25 @@ import org.gradle.internal.Factories;
 import org.gradle.internal.TrueTimeProvider;
 import org.gradle.internal.event.ListenerBroadcast;
 import org.gradle.internal.event.ListenerManager;
+import org.gradle.internal.operations.DefaultBuildOperationWorkerRegistry;
 import org.gradle.internal.progress.BuildOperationExecutor;
+import org.gradle.test.fixtures.file.TestNameTestDirectoryProvider;
 import org.gradle.util.JUnit4GroovyMockery;
 import org.gradle.util.TestClosure;
+import org.gradle.util.TestUtil;
 import org.hamcrest.Description;
 import org.jmock.Expectations;
 import org.jmock.api.Invocation;
 import org.jmock.integration.junit4.JMock;
 import org.jmock.integration.junit4.JUnit4Mockery;
 import org.junit.Before;
+import org.junit.Rule;
 import org.junit.Test;
 import org.junit.runner.RunWith;
 
 import java.util.ArrayList;
 import java.util.List;
 
-import static org.gradle.util.TestUtil.createRootProject;
 import static org.gradle.util.TestUtil.toClosure;
 import static org.gradle.util.WrapUtil.toList;
 import static org.gradle.util.WrapUtil.toSet;
@@ -67,9 +74,12 @@ public class DefaultTaskGraphExecuterTest {
     ProjectInternal root;
     List<Task> executedTasks = new ArrayList<Task>();
 
+    @Rule
+    public TestNameTestDirectoryProvider temporaryFolder = new TestNameTestDirectoryProvider();
+
     @Before
     public void setUp() {
-        root = createRootProject();
+        root = TestUtil.create(temporaryFolder).rootProject();
         context.checking(new Expectations(){{
             one(listenerManager).createAnonymousBroadcaster(TaskExecutionGraphListener.class);
             will(returnValue(new ListenerBroadcast<TaskExecutionGraphListener>(TaskExecutionGraphListener.class)));
@@ -80,7 +90,7 @@ public class DefaultTaskGraphExecuterTest {
             allowing(cancellationToken).isCancellationRequested();
             allowing(buildOperationExecutor).getCurrentOperationId();
         }});
-        taskExecuter = new DefaultTaskGraphExecuter(listenerManager, new DefaultTaskPlanExecutor(), Factories.constant(executer), cancellationToken, new TrueTimeProvider(), buildOperationExecutor);
+        taskExecuter = new DefaultTaskGraphExecuter(listenerManager, new DefaultTaskPlanExecutor(new DefaultBuildOperationWorkerRegistry(1)), Factories.constant(executer), cancellationToken, new TrueTimeProvider(), buildOperationExecutor);
     }
 
     @Test
@@ -274,10 +284,29 @@ public class DefaultTaskGraphExecuterTest {
 
     @Test
     public void testExecutesWhenReadyClosureBeforeExecute() {
+        assertExecutesWhenReadyListenerBeforeExecute(new Action<TestClosure>() {
+            @Override
+            public void execute(TestClosure testClosure) {
+                taskExecuter.whenReady(toClosure(testClosure));
+            }
+        });
+    }
+
+    @Test
+    public void testExecutesWhenReadyActionBeforeExecute() {
+        assertExecutesWhenReadyListenerBeforeExecute(new Action<TestClosure>() {
+            @Override
+            public void execute(TestClosure testClosure) {
+                taskExecuter.whenReady(toAction(testClosure));
+            }
+        });
+    }
+
+    void assertExecutesWhenReadyListenerBeforeExecute(Action<TestClosure> whenReadySubscriber) {
         final TestClosure runnable = context.mock(TestClosure.class);
         Task a = task("a");
 
-        taskExecuter.whenReady(toClosure(runnable));
+        whenReadySubscriber.execute(runnable);
 
         taskExecuter.addTasks(toList(a));
 
@@ -334,13 +363,30 @@ public class DefaultTaskGraphExecuterTest {
 
     @Test
     public void testNotifiesBeforeTaskClosureAsTasksAreExecuted() {
+        assertNotifiesBeforeTaskListenerAsTasksAreExecuted(new Action<TestClosure>() {
+            @Override
+            public void execute(TestClosure testClosure) {
+                taskExecuter.beforeTask(toClosure(testClosure));
+            }
+        });
+    }
+
+    @Test
+    public void testNotifiesBeforeTaskActionAsTasksAreExecuted() {
+        assertNotifiesBeforeTaskListenerAsTasksAreExecuted(new Action<TestClosure>() {
+            @Override
+            public void execute(TestClosure testClosure) {
+                taskExecuter.beforeTask(toAction(testClosure));
+            }
+        });
+    }
+
+    void assertNotifiesBeforeTaskListenerAsTasksAreExecuted(Action<TestClosure> beforeTaskSubscriber) {
         final TestClosure runnable = context.mock(TestClosure.class);
+        beforeTaskSubscriber.execute(runnable);
+
         final Task a = task("a");
         final Task b = task("b");
-
-        final Closure closure = toClosure(runnable);
-        taskExecuter.beforeTask(closure);
-
         taskExecuter.addTasks(toList(a, b));
 
         context.checking(new Expectations() {{
@@ -353,12 +399,30 @@ public class DefaultTaskGraphExecuterTest {
 
     @Test
     public void testNotifiesAfterTaskClosureAsTasksAreExecuted() {
+        assertNotifiesAfterTaskListenerAsTasksAreExecuted(new Action<TestClosure>() {
+            @Override
+            public void execute(TestClosure testClosure) {
+                taskExecuter.afterTask(toClosure(testClosure));
+            }
+        });
+    }
+
+    @Test
+    public void testNotifiesAfterTaskActionAsTasksAreExecuted() {
+        assertNotifiesAfterTaskListenerAsTasksAreExecuted(new Action<TestClosure>() {
+            @Override
+            public void execute(TestClosure testClosure) {
+                taskExecuter.afterTask(toAction(testClosure));
+            }
+        });
+    }
+
+    void assertNotifiesAfterTaskListenerAsTasksAreExecuted(Action<TestClosure> afterTaskSubscriber) {
         final TestClosure runnable = context.mock(TestClosure.class);
+        afterTaskSubscriber.execute(runnable);
+
         final Task a = task("a");
         final Task b = task("b");
-
-        taskExecuter.afterTask(toClosure(runnable));
-
         taskExecuter.addTasks(toList(a, b));
 
         context.checking(new Expectations() {{
@@ -437,6 +501,15 @@ public class DefaultTaskGraphExecuterTest {
         assertThat(executedTasks, equalTo(toList(a, c)));
     }
 
+    static Action toAction(final TestClosure closure) {
+        return new Action() {
+            @Override
+            public void execute(Object o) {
+                closure.call(o);
+            }
+        };
+    }
+
     private void dependsOn(final Task task, final Task... dependsOn) {
         context.checking(new Expectations() {{
             TaskDependency taskDependency = context.mock(TaskDependency.class);
