@@ -24,24 +24,25 @@ import org.gradle.api.Project;
 import org.gradle.api.ProjectEvaluationListener;
 import org.gradle.api.initialization.dsl.ScriptHandler;
 import org.gradle.api.internal.AsmBackedClassGenerator;
-import org.gradle.api.internal.GradleDistributionLocator;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.initialization.ClassLoaderScope;
 import org.gradle.api.internal.initialization.ScriptHandlerFactory;
-import org.gradle.api.plugins.PluginManager;
 import org.gradle.api.internal.plugins.DefaultPluginManager;
 import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.invocation.Gradle;
 import org.gradle.api.plugins.PluginContainer;
+import org.gradle.api.plugins.PluginManager;
 import org.gradle.configuration.ScriptPluginFactory;
 import org.gradle.execution.TaskGraphExecuter;
 import org.gradle.initialization.ClassLoaderScopeRegistry;
 import org.gradle.internal.classloader.MultiParentClassLoader;
+import org.gradle.internal.event.ListenerBroadcast;
+import org.gradle.internal.event.ListenerManager;
+import org.gradle.internal.installation.CurrentGradleInstallation;
+import org.gradle.internal.installation.GradleInstallation;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
 import org.gradle.listener.ClosureBackedMethodInvocationDispatch;
-import org.gradle.internal.event.ListenerBroadcast;
-import org.gradle.internal.event.ListenerManager;
 import org.gradle.util.GradleVersion;
 import org.gradle.util.JUnit4GroovyMockery;
 import org.gradle.util.TestUtil;
@@ -72,7 +73,7 @@ public class DefaultGradleTest {
     private final ListenerManager listenerManager = context.mock(ListenerManager.class);
     private final Gradle parent = context.mock(Gradle.class, "parentBuild");
     private final MultiParentClassLoader scriptClassLoaderMock = context.mock(MultiParentClassLoader.class);
-    private final GradleDistributionLocator gradleDistributionLocatorMock = context.mock(GradleDistributionLocator.class);
+    private final CurrentGradleInstallation currentGradleInstallationMock = context.mock(CurrentGradleInstallation.class);
     private final ListenerBroadcast<BuildListener> buildListenerBroadcast = new ListenerBroadcast<BuildListener>(BuildListener.class);
     private final ListenerBroadcast<ProjectEvaluationListener> projectEvaluationListenerBroadcast = context.mock(ListenerBroadcast.class);
     private final FileResolver fileResolverMock = context.mock(FileResolver.class);
@@ -90,29 +91,29 @@ public class DefaultGradleTest {
         context.checking(new Expectations() {{
             one(serviceRegistryFactoryMock).createFor(with(any(DefaultGradle.class)));
             will(returnValue(gradleServiceRegistryMock));
-            allowing(gradleServiceRegistryMock).get((Type)ScriptHandler.class);
+            allowing(gradleServiceRegistryMock).get((Type) ScriptHandler.class);
             will(returnValue(scriptHandlerMock));
             allowing(gradleServiceRegistryMock).get(ClassLoaderScopeRegistry.class);
             will(returnValue(classLoaderScopeRegistry));
             allowing(classLoaderScopeRegistry).getCoreAndPluginsScope();
             will(returnValue(classLoaderScope));
-            allowing(gradleServiceRegistryMock).get((Type)DefaultPluginManager.class);
+            allowing(gradleServiceRegistryMock).get((Type) DefaultPluginManager.class);
             will(returnValue(pluginManager));
-            allowing(gradleServiceRegistryMock).get((Type)TaskGraphExecuter.class);
+            allowing(gradleServiceRegistryMock).get((Type) TaskGraphExecuter.class);
             will(returnValue(taskExecuter));
-            allowing(gradleServiceRegistryMock).get((Type)ListenerManager.class);
+            allowing(gradleServiceRegistryMock).get((Type) ListenerManager.class);
             will(returnValue(listenerManager));
             allowing(gradleServiceRegistryMock).get(MultiParentClassLoader.class);
             will(returnValue(scriptClassLoaderMock));
-            allowing(gradleServiceRegistryMock).get((Type)GradleDistributionLocator.class);
-            will(returnValue(gradleDistributionLocatorMock));
+            allowing(gradleServiceRegistryMock).get((Type) CurrentGradleInstallation.class);
+            will(returnValue(currentGradleInstallationMock));
             allowing(gradleServiceRegistryMock).get(PluginContainer.class);
             will(returnValue(pluginContainer));
-            allowing(gradleServiceRegistryMock).get((Type)FileResolver.class);
+            allowing(gradleServiceRegistryMock).get((Type) FileResolver.class);
             will(returnValue(fileResolverMock));
-            allowing(gradleServiceRegistryMock).get((Type)ScriptPluginFactory.class);
+            allowing(gradleServiceRegistryMock).get((Type) ScriptPluginFactory.class);
             will(returnValue(scriptPluginFactory));
-            allowing(gradleServiceRegistryMock).get((Type)ScriptHandlerFactory.class);
+            allowing(gradleServiceRegistryMock).get((Type) ScriptHandlerFactory.class);
             will(returnValue(scriptHandlerFactory));
             allowing(listenerManager).createAnonymousBroadcaster(BuildListener.class);
             will(returnValue(buildListenerBroadcast));
@@ -141,8 +142,8 @@ public class DefaultGradleTest {
         final File gradleHome = new File("home");
 
         context.checking(new Expectations() {{
-            one(gradleDistributionLocatorMock).getGradleHome();
-            will(returnValue(gradleHome));
+            one(currentGradleInstallationMock).getInstallation();
+            will(returnValue(new GradleInstallation(gradleHome)));
         }});
 
         assertThat(gradle.getGradleHomeDir(), equalTo(gradleHome));
