@@ -17,54 +17,85 @@ package org.gradle.integtests.fixtures.executer;
 
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Sets;
+import com.google.common.io.CharSource;
 import groovy.lang.Closure;
 import groovy.lang.DelegatesTo;
 import org.gradle.api.Action;
 import org.gradle.api.Transformer;
+import org.gradle.api.UncheckedIOException;
 import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.initialization.DefaultClassLoaderScope;
+import org.gradle.api.internal.initialization.loadercache.ClassLoaderCache;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
+import org.gradle.integtests.fixtures.daemon.DaemonLogsAnalyzer;
 import org.gradle.internal.UncheckedException;
+import org.gradle.internal.concurrent.CompositeStoppable;
+import org.gradle.internal.featurelifecycle.LoggingDeprecatedFeatureHandler;
 import org.gradle.internal.jvm.Jvm;
+import org.gradle.internal.logging.services.LoggingServiceRegistry;
+import org.gradle.internal.nativeintegration.services.NativeServices;
+import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.internal.service.ServiceRegistryBuilder;
+import org.gradle.internal.service.scopes.GlobalScopeServices;
 import org.gradle.launcher.daemon.configuration.DaemonParameters;
 import org.gradle.launcher.daemon.configuration.GradleProperties;
 import org.gradle.listener.ActionBroadcast;
 import org.gradle.process.internal.streams.SafeStreams;
 import org.gradle.test.fixtures.file.TestDirectoryProvider;
 import org.gradle.test.fixtures.file.TestFile;
+import org.gradle.testfixtures.internal.NativeServicesTestFixture;
 import org.gradle.util.CollectionUtils;
-import org.gradle.util.DeprecationLogger;
+import org.gradle.util.GradleVersion;
 
-import java.io.*;
+import java.io.File;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.PipedInputStream;
+import java.io.PipedOutputStream;
 import java.nio.charset.Charset;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.HashSet;
+import java.util.LinkedHashMap;
+import java.util.LinkedHashSet;
+import java.util.List;
+import java.util.Locale;
+import java.util.Map;
+import java.util.Set;
 
-import static org.gradle.launcher.daemon.client.DefaultDaemonConnector.DISABLE_STARTING_DAEMON_MESSAGE_PROPERTY;
+import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.*;
+import static org.gradle.integtests.fixtures.executer.OutputScrapingExecutionResult.STACK_TRACE_ELEMENT;
+import static org.gradle.internal.service.scopes.DefaultGradleUserHomeScopeServiceRegistry.REUSE_USER_HOME_SERVICES;
 import static org.gradle.util.CollectionUtils.collect;
 import static org.gradle.util.CollectionUtils.join;
-import static org.gradle.util.Matchers.containsLine;
-import static org.gradle.util.Matchers.matchesRegexp;
 
 public abstract class AbstractGradleExecuter implements GradleExecuter {
-    protected static Set<String> propagatedSystemProperties = Sets.newHashSet();
+    protected static final ServiceRegistry GLOBAL_SERVICES = ServiceRegistryBuilder.builder()
+        .displayName("Global services")
+        .parent(LoggingServiceRegistry.newCommandLineProcessLogging())
+        .parent(NativeServicesTestFixture.getInstance())
+        .provider(new GlobalScopeServices(true))
+        .build();
+    protected final static Set<String> PROPAGATED_SYSTEM_PROPERTIES = Sets.newHashSet();
 
     public static void propagateSystemProperty(String name) {
-        propagatedSystemProperties.add(name);
+        PROPAGATED_SYSTEM_PROPERTIES.add(name);
     }
 
     private static final String DEBUG_SYSPROP = "org.gradle.integtest.debug";
     private static final String PROFILE_SYSPROP = "org.gradle.integtest.profile";
 
     protected static final List<String> DEBUG_ARGS = ImmutableList.of(
-        "-Xdebug",
-        "-Xrunjdwp:transport=dt_socket,server=y,suspend=y,address=5005"
+        "-agentlib:jdwp=transport=dt_socket,server=y,suspend=y,address=5005"
     );
 
     private final Logger logger;
 
     protected final IntegrationTestBuildContext buildContext = new IntegrationTestBuildContext();
 
+    private final Set<File> customDaemonBaseDirs = new HashSet<File>();
     private final List<String> args = new ArrayList<String>();
     private final List<String> tasks = new ArrayList<String>();
     private boolean allowExtraLogging = true;
@@ -85,16 +116,15 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private PipedOutputStream stdinPipe;
     private String defaultCharacterEncoding;
     private Locale defaultLocale;
-    private int daemonIdleTimeoutSecs = 60;
+    private int daemonIdleTimeoutSecs = 120;
     private boolean requireDaemon;
     private File daemonBaseDir = buildContext.getDaemonBaseDir();
     private final List<String> buildJvmOpts = new ArrayList<String>();
     private final List<String> commandLineJvmOpts = new ArrayList<String>();
     private boolean useOnlyRequestedJvmOpts;
-    private boolean requireGradleHome;
-    private boolean daemonStartingMessageDisabled = true;
+    private boolean requiresGradleDistribution;
 
-    private boolean deprecationChecksOn = true;
+    private int expectedDeprecationWarnings;
     private boolean eagerClassLoaderCreationChecksOn = true;
     private boolean stackTraceChecksOn = true;
 
@@ -102,6 +132,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private final Set<Action<? super GradleExecuter>> afterExecute = new LinkedHashSet<Action<? super GradleExecuter>>();
 
     private final TestDirectoryProvider testDirectoryProvider;
+    private final GradleVersion gradleVersion;
     private final GradleDistribution distribution;
 
     private boolean debug = Boolean.getBoolean(DEBUG_SYSPROP);
@@ -109,11 +140,19 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     protected boolean interactive;
 
-    protected boolean noExplicitTmpDir;
+    private boolean noExplicitTmpDir;
+    protected boolean noExplicitNativeServicesDir;
+    private boolean fullDeprecationStackTrace = true;
+    private boolean checkDeprecations = true;
 
     protected AbstractGradleExecuter(GradleDistribution distribution, TestDirectoryProvider testDirectoryProvider) {
+        this(distribution, testDirectoryProvider, GradleVersion.current());
+    }
+
+    protected AbstractGradleExecuter(GradleDistribution distribution, TestDirectoryProvider testDirectoryProvider, GradleVersion gradleVersion) {
         this.distribution = distribution;
         this.testDirectoryProvider = testDirectoryProvider;
+        this.gradleVersion = gradleVersion;
         logger = Logging.getLogger(getClass());
     }
 
@@ -142,11 +181,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         commandLineJvmOpts.clear();
         buildJvmOpts.clear();
         useOnlyRequestedJvmOpts = false;
-        deprecationChecksOn = true;
+        expectedDeprecationWarnings = 0;
         stackTraceChecksOn = true;
         debug = Boolean.getBoolean(DEBUG_SYSPROP);
         profiler = System.getProperty(PROFILE_SYSPROP, "");
         interactive = false;
+        checkDeprecations = true;
         return this;
     }
 
@@ -234,6 +274,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (noExplicitTmpDir) {
             executer.withNoExplicitTmpDir();
         }
+        if (noExplicitNativeServicesDir) {
+            executer.withNoExplicitNativeServicesDir();
+        }
+        if (!fullDeprecationStackTrace) {
+            executer.withFullDeprecationStackTraceDisabled();
+        }
         if (defaultLocale != null) {
             executer.withDefaultLocale(defaultLocale);
         }
@@ -244,8 +290,8 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
         executer.noExtraLogging();
 
-        if (!deprecationChecksOn) {
-            executer.withDeprecationChecksDisabled();
+        for (int i = 0; i < expectedDeprecationWarnings; i++) {
+            executer.expectDeprecationWarning();
         }
         if (!eagerClassLoaderCreationChecksOn) {
             executer.withEagerClassLoaderCreationCheckDisabled();
@@ -253,19 +299,21 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (!stackTraceChecksOn) {
             executer.withStackTraceChecksDisabled();
         }
-        if (requireGradleHome) {
-            executer.requireGradleHome();
-        }
-        if (!daemonStartingMessageDisabled) {
-            executer.withDaemonStartingMessageEnabled();
+        if (requiresGradleDistribution) {
+            executer.requireGradleDistribution();
         }
         if (requireDaemon) {
             executer.requireDaemon();
         }
 
-        executer.withDebug(debug);
+        executer.startBuildProcessInDebugger(debug);
         executer.withProfiler(profiler);
         executer.withForceInteractive(interactive);
+
+        if (!checkDeprecations) {
+            executer.noDeprecationChecks();
+        }
+
         return executer;
     }
 
@@ -337,7 +385,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
      * Adjusts the calculated invocation prior to execution. This method is responsible for handling the implicit launcher JVM args in some way, by mutating the invocation appropriately.
      */
     protected void transformInvocation(GradleInvocation gradleInvocation) {
-        gradleInvocation.launcherJvmArgs.addAll(gradleInvocation.implicitLauncherJvmArgs);
+        gradleInvocation.launcherJvmArgs.addAll(0, gradleInvocation.implicitLauncherJvmArgs);
         gradleInvocation.implicitLauncherJvmArgs.clear();
     }
 
@@ -377,6 +425,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     protected List<String> getImplicitBuildJvmArgs() {
         List<String> buildJvmOpts = new ArrayList<String>();
         buildJvmOpts.add("-ea");
+
         if (isDebug()) {
             buildJvmOpts.addAll(DEBUG_ARGS);
         }
@@ -545,6 +594,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return withDaemonBaseDir(testDirectoryProvider.getTestDirectory().file("daemon"));
     }
 
+    public boolean usesSharedDaemons() {
+        return isSharedDaemons();
+    }
+
     public File getDaemonBaseDir() {
         return daemonBaseDir;
     }
@@ -560,17 +613,55 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     }
 
     protected boolean isUseDaemon() {
-        for (int i = args.size() - 1; i >= 0; i--) {
-            if (args.get(i).equals("--daemon")) {
-                return true;
-            }
-            if (args.get(i).equals("--no-daemon")) {
-                return false;
-            }
+        CliDaemonArgument cliDaemonArgument = resolveCliDaemonArgument();
+        if (cliDaemonArgument == DAEMON) {
+            return true;
         }
         return requireDaemon;
     }
 
+    /**
+     * Performs cleanup at completion of the test.
+     */
+    public void cleanup() {
+        for (File baseDir : customDaemonBaseDirs) {
+            new DaemonLogsAnalyzer(baseDir, gradleVersion.getVersion()).killAll();
+        }
+    }
+
+    // gets called by reflection from AbstractTestDirectoryProvider#closeCachedClassLoaders method
+    public static void cleanupCachedClassLoaders() {
+        ClassLoaderCache classLoaderCache = GLOBAL_SERVICES.get(ClassLoaderCache.class);
+        CompositeStoppable.stoppable(classLoaderCache).stop();
+    }
+
+    enum CliDaemonArgument {
+        NOT_DEFINED,
+        DAEMON,
+        NO_DAEMON,
+        FOREGROUND
+    }
+
+    private CliDaemonArgument resolveCliDaemonArgument() {
+        for (int i = args.size() - 1; i >= 0; i--) {
+            final String arg = args.get(i);
+            if (arg.equals("--daemon")) {
+                return DAEMON;
+            }
+            if (arg.equals("--no-daemon")) {
+                return NO_DAEMON;
+            }
+            if (arg.equals("--foreground")) {
+                return FOREGROUND;
+            }
+        }
+        return NOT_DEFINED;
+    }
+
+    private boolean noDaemonArgumentGiven() {
+        return resolveCliDaemonArgument() == NOT_DEFINED;
+    }
+
     protected List<String> getAllArgs() {
         List<String> allArgs = new ArrayList<String>();
         if (buildScript != null) {
@@ -592,8 +683,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (quiet) {
             allArgs.add("--quiet");
         }
-        if (isUseDaemon()) {
-            allArgs.add("--daemon");
+        if (noDaemonArgumentGiven()) {
+            if (isUseDaemon()) {
+                allArgs.add("--daemon");
+            } else {
+                allArgs.add("--no-daemon");
+            }
         }
         allArgs.add("--stacktrace");
         if (taskList) {
@@ -646,7 +741,14 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
         properties.put(GradleProperties.IDLE_TIMEOUT_PROPERTY, "" + (daemonIdleTimeoutSecs * 1000));
         properties.put(GradleProperties.DAEMON_BASE_DIR_PROPERTY, daemonBaseDir.getAbsolutePath());
-        properties.put(DeprecationLogger.ORG_GRADLE_DEPRECATION_TRACE_PROPERTY_NAME, "true");
+        if (!noExplicitNativeServicesDir) {
+            properties.put(NativeServices.NATIVE_DIR_OVERRIDE, buildContext.getNativeServicesDir().getAbsolutePath());
+        }
+        properties.put(LoggingDeprecatedFeatureHandler.ORG_GRADLE_DEPRECATION_TRACE_PROPERTY_NAME, Boolean.toString(fullDeprecationStackTrace));
+
+        if (gradleUserHomeDir != null && !gradleUserHomeDir.equals(buildContext.getGradleUserHomeDir())) {
+            properties.put(REUSE_USER_HOME_SERVICES, "false");
+        }
 
         if (!noExplicitTmpDir) {
             String tmpDirPath = getDefaultTmpDir().createDir().getAbsolutePath();
@@ -671,10 +773,6 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
             properties.put(DaemonParameters.INTERACTIVE_TOGGLE, "true");
         }
 
-        if (daemonStartingMessageDisabled) {
-            properties.put(DISABLE_STARTING_DAEMON_MESSAGE_PROPERTY, "true");
-        }
-
         return properties;
     }
 
@@ -686,6 +784,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         assert afterExecute.isEmpty() : "afterExecute actions are not implemented for async execution";
         fireBeforeExecute();
         assertCanExecute();
+        collectStateBeforeExecution();
         try {
             return doStart();
         } finally {
@@ -696,6 +795,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     public final ExecutionResult run() {
         fireBeforeExecute();
         assertCanExecute();
+        collectStateBeforeExecution();
         try {
             return doRun();
         } finally {
@@ -714,6 +814,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     public final ExecutionFailure runWithFailure() {
         fireBeforeExecute();
         assertCanExecute();
+        collectStateBeforeExecution();
         try {
             return doRunWithFailure();
         } finally {
@@ -721,6 +822,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
     }
 
+    private void collectStateBeforeExecution() {
+        if (!isSharedDaemons()) {
+            customDaemonBaseDirs.add(daemonBaseDir);
+        }
+    }
+
     private void fireBeforeExecute() {
         beforeExecute.execute(this);
     }
@@ -758,56 +865,65 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     }
 
     protected Action<ExecutionResult> getResultAssertion() {
-        ActionBroadcast<ExecutionResult> assertions = new ActionBroadcast<ExecutionResult>();
+        return new Action<ExecutionResult>() {
+            int expectedDeprecationWarnings = AbstractGradleExecuter.this.expectedDeprecationWarnings;
+            boolean expectStackTraces = !AbstractGradleExecuter.this.stackTraceChecksOn;
+            boolean checkDeprecations = AbstractGradleExecuter.this.checkDeprecations;
 
-        if (stackTraceChecksOn) {
-            assertions.add(new Action<ExecutionResult>() {
-                public void execute(ExecutionResult executionResult) {
-                    assertNoStackTraces(executionResult.getOutput(), "Standard output");
+            @Override
+            public void execute(ExecutionResult executionResult) {
+                validate(executionResult.getNormalizedOutput(), "Standard output");
+                String error = executionResult.getError();
+                if (executionResult instanceof ExecutionFailure) {
+                    // Axe everything after the expected exception
+                    int pos = error.indexOf("* Exception is:\n");
+                    if (pos >= 0) {
+                        error = error.substring(0, pos);
+                    }
+                }
+                validate(error, "Standard error");
+            }
 
-                    String error = executionResult.getError();
-                    if (executionResult instanceof ExecutionFailure) {
-                        // Axe everything after the expected exception
-                        int pos = error.indexOf("* Exception is:\n");
-                        if (pos >= 0) {
-                            error = error.substring(0, pos);
+            private void validate(String output, String displayName) {
+                List<String> lines;
+                try {
+                    lines = CharSource.wrap(output).readLines();
+                } catch (IOException e) {
+                    throw new UncheckedIOException(e);
+                }
+                int i = 0;
+                while (i < lines.size()) {
+                    String line = lines.get(i);
+                    if (line.matches(".*use(s)? or override(s)? a deprecated API\\.")) {
+                        // A javac warning, ignore
+                        i++;
+                    } else if (line.matches(".*\\s+deprecated.*")) {
+                        if (checkDeprecations && expectedDeprecationWarnings <= 0) {
+                            throw new AssertionError(String.format("%s line %d contains a deprecation warning: %s%n=====%n%s%n=====%n", displayName, i+1, line, output));
                         }
-                    }
-                    assertNoStackTraces(error, "Standard error");
-                }
-
-                private void assertNoStackTraces(String output, String displayName) {
-                    if (containsLine(matchesRegexp("\\s+(at\\s+)?[\\w.$_]+\\([\\w._]+:\\d+\\)")).matches(output)) {
-                        throw new AssertionError(String.format("%s contains an unexpected stack trace:%n=====%n%s%n=====%n", displayName, output));
+                        expectedDeprecationWarnings--;
+                        // skip over stack trace
+                        i++;
+                        while (i < lines.size() && STACK_TRACE_ELEMENT.matcher(lines.get(i)).matches()) {
+                            i++;
+                        }
+                    } else if (!expectStackTraces && STACK_TRACE_ELEMENT.matcher(line).matches() && i < lines.size()-1 && STACK_TRACE_ELEMENT.matcher(lines.get(i+1)).matches()) {
+                        // 2 or more lines that look like stack trace elements
+                        throw new AssertionError(String.format("%s line %d contains an unexpected stack trace: %s%n=====%n%s%n=====%n", displayName, i+1, line, output));
+                    } else {
+                        i++;
                     }
                 }
-            });
-        }
-
-        if (deprecationChecksOn) {
-            assertions.add(new Action<ExecutionResult>() {
-                public void execute(ExecutionResult executionResult) {
-                    assertNoDeprecationWarnings(executionResult.getOutput(), "Standard output");
-                    assertNoDeprecationWarnings(executionResult.getError(), "Standard error");
-                }
-
-                private void assertNoDeprecationWarnings(String output, String displayName) {
-                    boolean javacWarning = containsLine(matchesRegexp(".*use(s)? or override(s)? a deprecated API\\.")).matches(output);
-                    boolean deprecationWarning = containsLine(matchesRegexp(".* deprecated.*")).matches(output);
-                    if (deprecationWarning && !javacWarning) {
-                        throw new AssertionError(String.format("%s contains a deprecation warning:%n=====%n%s%n=====%n", displayName, output));
-                    }
-                }
-            });
-        }
-
-        return assertions;
+            }
+        };
     }
 
-    public GradleExecuter withDeprecationChecksDisabled() {
-        deprecationChecksOn = false;
-        // turn off stack traces too
-        stackTraceChecksOn = false;
+    public GradleExecuter expectDeprecationWarning() {
+        expectedDeprecationWarnings++;
+        return this;
+    }
+    public GradleExecuter noDeprecationChecks() {
+        checkDeprecations = false;
         return this;
     }
 
@@ -834,22 +950,17 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return allowExtraLogging;
     }
 
-    public boolean isRequireGradleHome() {
-        return requireGradleHome;
+    public boolean isRequiresGradleDistribution() {
+        return requiresGradleDistribution;
     }
 
-    public GradleExecuter requireGradleHome() {
-        this.requireGradleHome = true;
-        return this;
-    }
-
-    public GradleExecuter withDaemonStartingMessageEnabled() {
-        daemonStartingMessageDisabled = false;
+    public GradleExecuter requireGradleDistribution() {
+        this.requiresGradleDistribution = true;
         return this;
     }
 
     @Override
-    public GradleExecuter withDebug(boolean flag) {
+    public GradleExecuter startBuildProcessInDebugger(boolean flag) {
         debug = flag;
         return this;
     }
@@ -871,6 +982,17 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
+    @Override
+    public GradleExecuter withNoExplicitNativeServicesDir() {
+        noExplicitNativeServicesDir = true;
+        return this;
+    }
+
+    public GradleExecuter withFullDeprecationStackTraceDisabled() {
+        fullDeprecationStackTrace = false;
+        return this;
+    }
+
     @Override
     public boolean isDebug() {
         return debug;
