@@ -23,8 +23,11 @@ import org.gradle.test.fixtures.ConcurrentTestUtil;
 import org.junit.rules.TestRule;
 import org.junit.runner.Description;
 import org.junit.runners.model.Statement;
+import org.slf4j.Logger;
+import org.slf4j.LoggerFactory;
 
 import java.io.IOException;
+import java.lang.reflect.Method;
 import java.util.Random;
 import java.util.regex.Pattern;
 
@@ -33,7 +36,7 @@ import java.util.regex.Pattern;
  * A JUnit rule which provides a unique temporary folder for the test.
  */
 abstract class AbstractTestDirectoryProvider implements TestRule, TestDirectoryProvider {
-
+    private static final Logger LOG = LoggerFactory.getLogger(AbstractTestDirectoryProvider.class);
     protected static TestFile root;
 
     private static final Random RANDOM = new Random();
@@ -66,10 +69,7 @@ abstract class AbstractTestDirectoryProvider implements TestRule, TestDirectoryP
         init(description.getMethodName(), testClass.getSimpleName());
 
         suppressCleanupErrors = testClass.getAnnotation(LeaksFileHandles.class) != null
-            || description.getAnnotation(LeaksFileHandles.class) != null
-            // For now, assume that all tests run with the daemon executer leak file handles
-            // This seems to be true for any test that uses `GradleExecuter.requireOwnGradleUserHomeDir`
-            || "daemon".equals(System.getProperty("org.gradle.integtest.executer"));
+            || description.getAnnotation(LeaksFileHandles.class) != null;
 
         return new TestDirectoryCleaningStatement(base, description.getDisplayName());
     }
@@ -93,7 +93,12 @@ abstract class AbstractTestDirectoryProvider implements TestRule, TestDirectoryP
                     ConcurrentTestUtil.poll(new Closure(null, null) {
                         @SuppressWarnings("UnusedDeclaration")
                         void doCall() throws IOException {
-                            FileUtils.forceDelete(dir);
+                            try {
+                                FileUtils.forceDelete(dir);
+                            } catch(IOException e) {
+                                closeCachedClassLoaders();
+                                throw e;
+                            }
                         }
                     });
                 }
@@ -108,6 +113,18 @@ abstract class AbstractTestDirectoryProvider implements TestRule, TestDirectoryP
         }
     }
 
+    // use reflection to close cached classloaders in AbstractGradleExecuter
+    private static void closeCachedClassLoaders() {
+        try {
+            Class<?> abstractGradleExecuterClazz = Class.forName("org.gradle.integtests.fixtures.executer.AbstractGradleExecuter", false, AbstractTestDirectoryProvider.class.getClassLoader());
+            Method cleanupMethod = abstractGradleExecuterClazz.getMethod("cleanupCachedClassLoaders");
+            cleanupMethod.invoke(null);
+        } catch (Exception e) {
+            // swallow exception, just log a warning
+            LOG.warn("Cannot close cached classloaders", e);
+        }
+    }
+
     protected void init(String methodName, String className) {
         if (methodName == null) {
             // must be a @ClassRule; use the rule's class name instead
@@ -124,26 +141,34 @@ abstract class AbstractTestDirectoryProvider implements TestRule, TestDirectoryP
 
     public TestFile getTestDirectory() {
         if (dir == null) {
-            if (prefix == null) {
-                // This can happen if this is used in a constructor or a @Before method. It also happens when using
-                // @RunWith(SomeRunner) when the runner does not support rules.
-                prefix = determinePrefix();
-            }
-            while (true) {
-                // Use a random prefix to avoid reusing test directories
-                String prefix = Integer.toString(RANDOM.nextInt(MAX_RANDOM_PART_VALUE), ALL_DIGITS_AND_LETTERS_RADIX);
-                if (WINDOWS_RESERVED_NAMES.matcher(prefix).matches()) {
-                    continue;
-                }
-                dir = root.file(this.prefix, prefix);
-                if (dir.mkdirs()) {
-                    break;
-                }
-            }
+           dir = createUniqueTestDirectory();
         }
         return dir;
     }
 
+    private TestFile createUniqueTestDirectory() {
+        while (true) {
+            // Use a random prefix to avoid reusing test directories
+            String randomPrefix = Integer.toString(RANDOM.nextInt(MAX_RANDOM_PART_VALUE), ALL_DIGITS_AND_LETTERS_RADIX);
+            if (WINDOWS_RESERVED_NAMES.matcher(randomPrefix).matches()) {
+                continue;
+            }
+            TestFile dir = root.file(getPrefix(), randomPrefix);
+            if (dir.mkdirs()) {
+                return dir;
+            }
+        }
+    }
+
+    private String getPrefix() {
+        if (prefix == null) {
+            // This can happen if this is used in a constructor or a @Before method. It also happens when using
+            // @RunWith(SomeRunner) when the runner does not support rules.
+            prefix = determinePrefix();
+        }
+        return prefix;
+    }
+
     public TestFile file(Object... path) {
         return getTestDirectory().file((Object[]) path);
     }
