@@ -19,7 +19,19 @@ package org.gradle.api.tasks.javadoc;
 import groovy.lang.Closure;
 import org.gradle.api.Incubating;
 import org.gradle.api.file.FileCollection;
-import org.gradle.api.tasks.*;
+import org.gradle.api.file.FileTree;
+import org.gradle.api.tasks.CacheableTask;
+import org.gradle.api.tasks.Classpath;
+import org.gradle.api.tasks.Input;
+import org.gradle.api.tasks.Internal;
+import org.gradle.api.tasks.Nested;
+import org.gradle.api.tasks.Optional;
+import org.gradle.api.tasks.OutputDirectory;
+import org.gradle.api.tasks.ParallelizableTask;
+import org.gradle.api.tasks.PathSensitive;
+import org.gradle.api.tasks.PathSensitivity;
+import org.gradle.api.tasks.SourceTask;
+import org.gradle.api.tasks.TaskAction;
 import org.gradle.api.tasks.javadoc.internal.JavadocSpec;
 import org.gradle.external.javadoc.MinimalJavadocOptions;
 import org.gradle.external.javadoc.StandardJavadocDocletOptions;
@@ -71,6 +83,7 @@ import java.util.List;
  * }
  * </pre>
  */
+@CacheableTask
 @ParallelizableTask
 public class Javadoc extends SourceTask {
     private File destinationDir;
@@ -143,6 +156,15 @@ public class Javadoc extends SourceTask {
         generator.execute(spec);
     }
 
+    /**
+     * {@inheritDoc}
+     */
+    @PathSensitive(PathSensitivity.RELATIVE)
+    @Override
+    public FileTree getSource() {
+        return super.getSource();
+    }
+
     /**
      * Returns the tool chain that will be used to generate the Javadoc.
      */
@@ -161,6 +183,7 @@ public class Javadoc extends SourceTask {
         throw new UnsupportedOperationException();
     }
 
+    @Internal
     private JavaPlatform getPlatform() {
         return DefaultJavaPlatform.current();
     }
@@ -170,11 +193,20 @@ public class Javadoc extends SourceTask {
      *
      * @return The directory.
      */
-    @OutputDirectory
+    @Internal
     public File getDestinationDir() {
         return destinationDir;
     }
 
+    @OutputDirectory
+    protected File getOutputDirectory() {
+        File destinationDir = getDestinationDir();
+        if (destinationDir == null) {
+            destinationDir = options.getDestinationDirectory();
+        }
+        return destinationDir;
+    }
+
     /**
      * <p>Sets the directory to generate the documentation into.</p>
      */
@@ -185,6 +217,7 @@ public class Javadoc extends SourceTask {
     /**
      * Returns the amount of memory allocated to this task.
      */
+    @Internal
     public String getMaxMemory() {
         return maxMemory;
     }
@@ -221,6 +254,7 @@ public class Javadoc extends SourceTask {
      *
      * @see #setVerbose(boolean)
      */
+    @Internal
     public boolean isVerbose() {
         return options.isVerbose();
     }
@@ -242,7 +276,7 @@ public class Javadoc extends SourceTask {
      *
      * @return The classpath.
      */
-    @InputFiles
+    @Classpath
     public FileCollection getClasspath() {
         return classpath;
     }
@@ -297,6 +331,7 @@ public class Javadoc extends SourceTask {
         this.failOnError = failOnError;
     }
 
+    @Internal
     public File getOptionsFile() {
         return new File(getTemporaryDir(), "javadoc.options");
     }
