@@ -104,56 +104,56 @@ public class StandardJavadocDocletOptionsTest {
 
         context.checking(new Expectations(){{
             // core options
-            one(optionFileMock).addStringOption("overview");
-            one(optionFileMock).addEnumOption("memberLevel");
-            one(optionFileMock).addStringOption("doclet");
-            one(optionFileMock).addPathOption("docletclasspath");
-            one(optionFileMock).addStringOption("source");
-            one(optionFileMock).addPathOption("sourcepath");
-            one(optionFileMock).addPathOption("classpath");
-            one(optionFileMock).addStringsOption("subpackages", ";");
-            one(optionFileMock).addStringsOption("exclude", ":");
-            one(optionFileMock).addPathOption("bootclasspath");
-            one(optionFileMock).addPathOption("extdirs");
-            one(optionFileMock).addEnumOption("outputLevel", JavadocOutputLevel.QUIET);
-            one(optionFileMock).addBooleanOption("breakiterator");
-            one(optionFileMock).addStringOption("locale");
-            one(optionFileMock).addStringOption("encoding");
+            oneOf(optionFileMock).addStringOption("overview");
+            oneOf(optionFileMock).addEnumOption("memberLevel");
+            oneOf(optionFileMock).addStringOption("doclet");
+            oneOf(optionFileMock).addPathOption("docletclasspath");
+            oneOf(optionFileMock).addStringOption("source");
+            oneOf(optionFileMock).addPathOption("sourcepath");
+            oneOf(optionFileMock).addPathOption("classpath");
+            oneOf(optionFileMock).addStringsOption("subpackages", ";");
+            oneOf(optionFileMock).addStringsOption("exclude", ":");
+            oneOf(optionFileMock).addPathOption("bootclasspath");
+            oneOf(optionFileMock).addPathOption("extdirs");
+            oneOf(optionFileMock).addEnumOption("outputLevel", JavadocOutputLevel.QUIET);
+            oneOf(optionFileMock).addBooleanOption("breakiterator");
+            oneOf(optionFileMock).addStringOption("locale");
+            oneOf(optionFileMock).addStringOption("encoding");
             // standard doclet options
-            one(optionFileMock).addFileOption("d");
-            one(optionFileMock).addBooleanOption("use");
-            one(optionFileMock).addBooleanOption("version");
-            one(optionFileMock).addBooleanOption("author");
-            one(optionFileMock).addBooleanOption("splitindex");
-            one(optionFileMock).addStringOption("windowtitle");
-            one(optionFileMock).addStringOption("doctitle");
-            one(optionFileMock).addStringOption("footer");
-            one(optionFileMock).addStringOption("bottom");
-            one(optionFileMock).addStringOption("link");
+            oneOf(optionFileMock).addFileOption("d");
+            oneOf(optionFileMock).addBooleanOption("use");
+            oneOf(optionFileMock).addBooleanOption("version");
+            oneOf(optionFileMock).addBooleanOption("author");
+            oneOf(optionFileMock).addBooleanOption("splitindex");
+            oneOf(optionFileMock).addStringOption("windowtitle");
+            oneOf(optionFileMock).addStringOption("doctitle");
+            oneOf(optionFileMock).addStringOption("footer");
+            oneOf(optionFileMock).addStringOption("bottom");
+            oneOf(optionFileMock).addStringOption("link");
             allowing(optionFileMock).addOption(new LinksOfflineJavadocOptionFileOption("linkoffline"));
-            one(optionFileMock).addBooleanOption("linksource");
-            one(optionFileMock).addOption(new GroupsJavadocOptionFileOption("group"));
-            one(optionFileMock).addBooleanOption("nodeprecated");
-            one(optionFileMock).addBooleanOption("nodeprecatedlist");
-            one(optionFileMock).addBooleanOption("nosince");
-            one(optionFileMock).addBooleanOption("notree");
-            one(optionFileMock).addBooleanOption("noindex");
-            one(optionFileMock).addBooleanOption("nohelp");
-            one(optionFileMock).addBooleanOption("nonavbar");
-            one(optionFileMock).addFileOption("helpfile");
-            one(optionFileMock).addFileOption("stylesheetfile");
-            one(optionFileMock).addBooleanOption("serialwarn");
-            one(optionFileMock).addStringOption("charset");
-            one(optionFileMock).addStringOption("docencoding");
-            one(optionFileMock).addBooleanOption("keywords");
-            one(optionFileMock).addStringOption("tags");
-            one(optionFileMock).addStringOption("taglets");
-            one(optionFileMock).addPathOption("tagletpath");
-            one(optionFileMock).addBooleanOption("docfilessubdirs");
-            one(optionFileMock).addStringsOption("excludedocfilessubdir", ":");
-            one(optionFileMock).addStringsOption("noqualifier", ":");
-            one(optionFileMock).addBooleanOption("notimestamp");
-            one(optionFileMock).addBooleanOption("nocomment");
+            oneOf(optionFileMock).addBooleanOption("linksource");
+            oneOf(optionFileMock).addOption(new GroupsJavadocOptionFileOption("group"));
+            oneOf(optionFileMock).addBooleanOption("nodeprecated");
+            oneOf(optionFileMock).addBooleanOption("nodeprecatedlist");
+            oneOf(optionFileMock).addBooleanOption("nosince");
+            oneOf(optionFileMock).addBooleanOption("notree");
+            oneOf(optionFileMock).addBooleanOption("noindex");
+            oneOf(optionFileMock).addBooleanOption("nohelp");
+            oneOf(optionFileMock).addBooleanOption("nonavbar");
+            oneOf(optionFileMock).addFileOption("helpfile");
+            oneOf(optionFileMock).addFileOption("stylesheetfile");
+            oneOf(optionFileMock).addBooleanOption("serialwarn");
+            oneOf(optionFileMock).addStringOption("charset");
+            oneOf(optionFileMock).addStringOption("docencoding");
+            oneOf(optionFileMock).addBooleanOption("keywords");
+            oneOf(optionFileMock).addStringOption("tags");
+            oneOf(optionFileMock).addStringOption("taglets");
+            oneOf(optionFileMock).addPathOption("tagletpath");
+            oneOf(optionFileMock).addBooleanOption("docfilessubdirs");
+            oneOf(optionFileMock).addStringsOption("excludedocfilessubdir", ":");
+            oneOf(optionFileMock).addStringsOption("noqualifier", ":");
+            oneOf(optionFileMock).addBooleanOption("notimestamp");
+            oneOf(optionFileMock).addBooleanOption("nocomment");
         }});
 
         options = new StandardJavadocDocletOptions();
