@@ -15,8 +15,14 @@
  */
 package org.gradle.internal.nativeintegration.services;
 
-import net.rubygrapefruit.platform.*;
+import net.rubygrapefruit.platform.NativeException;
+import net.rubygrapefruit.platform.NativeIntegrationUnavailableException;
+import net.rubygrapefruit.platform.PosixFiles;
 import net.rubygrapefruit.platform.Process;
+import net.rubygrapefruit.platform.ProcessLauncher;
+import net.rubygrapefruit.platform.SystemInfo;
+import net.rubygrapefruit.platform.Terminals;
+import net.rubygrapefruit.platform.WindowsRegistry;
 import net.rubygrapefruit.platform.internal.DefaultProcessLauncher;
 import org.gradle.internal.SystemProperties;
 import org.gradle.internal.jvm.Jvm;
@@ -27,7 +33,7 @@ import org.gradle.internal.nativeintegration.console.NoOpConsoleDetector;
 import org.gradle.internal.nativeintegration.console.WindowsConsoleDetector;
 import org.gradle.internal.nativeintegration.filesystem.services.FileSystemServices;
 import org.gradle.internal.nativeintegration.filesystem.services.UnavailablePosixFiles;
-import org.gradle.internal.nativeintegration.jna.JnaBootPathConfigurer;
+import org.gradle.internal.nativeintegration.jansi.JansiBootPathConfigurer;
 import org.gradle.internal.nativeintegration.jna.UnsupportedEnvironment;
 import org.gradle.internal.nativeintegration.processenvironment.NativePlatformBackedProcessEnvironment;
 import org.gradle.internal.os.OperatingSystem;
@@ -48,7 +54,9 @@ public class NativeServices extends DefaultServiceRegistry implements ServiceReg
     private static final Logger LOGGER = LoggerFactory.getLogger(NativeServices.class);
     private static boolean useNativePlatform = "true".equalsIgnoreCase(System.getProperty("org.gradle.native", "true"));
     private static final NativeServices INSTANCE = new NativeServices();
+    private static final JansiBootPathConfigurer JANSI_BOOT_PATH_CONFIGURER = new JansiBootPathConfigurer();
     private static boolean initialized;
+    private static File nativeBaseDir;
 
     public static final String NATIVE_DIR_OVERRIDE = "org.gradle.native.dir";
 
@@ -60,18 +68,12 @@ public class NativeServices extends DefaultServiceRegistry implements ServiceReg
         initialize(userHomeDir, true);
     }
 
-    public static synchronized void initialize(File userHomeDir, boolean initializeJNA) {
+    public static synchronized void initialize(File userHomeDir, boolean initializeJansi) {
         if (!initialized) {
-            String overrideProperty = System.getProperty(NATIVE_DIR_OVERRIDE);
-            File nativeDir;
-            if (overrideProperty == null) {
-                nativeDir = new File(userHomeDir, "native");
-            } else {
-                nativeDir = new File(overrideProperty);
-            }
+            nativeBaseDir = getNativeServicesDir(userHomeDir);
             if (useNativePlatform) {
                 try {
-                    net.rubygrapefruit.platform.Native.init(nativeDir);
+                    net.rubygrapefruit.platform.Native.init(nativeBaseDir);
                 } catch (NativeIntegrationUnavailableException ex) {
                     LOGGER.debug("Native-platform is not available.");
                     useNativePlatform = false;
@@ -84,14 +86,28 @@ public class NativeServices extends DefaultServiceRegistry implements ServiceReg
                     }
                 }
             }
-            if (OperatingSystem.current().isWindows() && initializeJNA) {
-                // JNA is still being used by jansi
-                new JnaBootPathConfigurer().configure(nativeDir);
+            if (initializeJansi) {
+                JANSI_BOOT_PATH_CONFIGURER.configure(nativeBaseDir);
             }
             initialized = true;
+
+            LOGGER.info("Initialized native services in: " + nativeBaseDir);
         }
     }
 
+    public static File getNativeServicesDir(File userHomeDir) {
+        String overrideProperty = getNativeDirOverride();
+        if (overrideProperty == null) {
+            return new File(userHomeDir, "native");
+        } else {
+            return new File(overrideProperty);
+        }
+    }
+
+    private static String getNativeDirOverride() {
+        return System.getProperty(NATIVE_DIR_OVERRIDE, System.getenv(NATIVE_DIR_OVERRIDE));
+    }
+
     public static synchronized NativeServices getInstance() {
         if (!initialized) {
             // If this occurs while running gradle or running integration tests, it is indicative of a problem.
@@ -216,6 +232,7 @@ public class NativeServices extends DefaultServiceRegistry implements ServiceReg
             this.type = type;
         }
 
+        @Override
         public Object invoke(Object proxy, Method method, Object[] args) throws Throwable {
             throw new org.gradle.internal.nativeintegration.NativeIntegrationUnavailableException(String.format("%s is not supported on this operating system.", type));
         }
