@@ -21,28 +21,34 @@ import com.amazonaws.AmazonServiceException;
 import com.amazonaws.ClientConfiguration;
 import com.amazonaws.auth.AWSCredentials;
 import com.amazonaws.auth.BasicAWSCredentials;
+import com.amazonaws.regions.Region;
 import com.amazonaws.services.s3.AmazonS3Client;
 import com.amazonaws.services.s3.S3ClientOptions;
-import com.amazonaws.services.s3.model.*;
+import com.amazonaws.services.s3.model.GetObjectRequest;
+import com.amazonaws.services.s3.model.ListObjectsRequest;
+import com.amazonaws.services.s3.model.ObjectListing;
+import com.amazonaws.services.s3.model.ObjectMetadata;
+import com.amazonaws.services.s3.model.PutObjectRequest;
+import com.amazonaws.services.s3.model.S3Object;
 import com.google.common.base.Optional;
+import com.google.common.collect.ImmutableList;
+import org.gradle.api.GradleException;
+import org.gradle.api.JavaVersion;
 import org.gradle.api.artifacts.repositories.PasswordCredentials;
 import org.gradle.api.credentials.AwsCredentials;
-import org.gradle.internal.resource.ResourceException;
+import org.gradle.internal.resource.ResourceExceptions;
 import org.gradle.internal.resource.transport.http.HttpProxySettings;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import java.io.InputStream;
 import java.net.URI;
-import java.util.ArrayList;
 import java.util.List;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
 public class S3Client {
     private static final Logger LOGGER = LoggerFactory.getLogger(S3Client.class);
-    private static final Pattern FILENAME_PATTERN = Pattern.compile("[^\\/]+\\.*$");
 
+    private S3ResourceResolver resourceResolver = new S3ResourceResolver();
     private AmazonS3Client amazonS3Client;
     private final S3ConnectionProperties s3ConnectionProperties;
 
@@ -59,16 +65,27 @@ public class S3Client {
 
     private AmazonS3Client createAmazonS3Client(AWSCredentials credentials) {
         AmazonS3Client amazonS3Client = new AmazonS3Client(credentials, createConnectionProperties());
-        S3ClientOptions clientOptions = new S3ClientOptions();
+        S3ClientOptions.Builder clientOptionsBuilder = S3ClientOptions.builder();
         Optional<URI> endpoint = s3ConnectionProperties.getEndpoint();
         if (endpoint.isPresent()) {
             amazonS3Client.setEndpoint(endpoint.get().toString());
-            clientOptions.withPathStyleAccess(true);
+            clientOptionsBuilder.setPathStyleAccess(true);
         }
-        amazonS3Client.setS3ClientOptions(clientOptions);
+        amazonS3Client.setS3ClientOptions(clientOptionsBuilder.build());
         return amazonS3Client;
     }
 
+    private void checkRequiredJigsawModuleIsOnPath() {
+        if (JavaVersion.current().isJava9Compatible()) {
+            try {
+                Class.forName("javax.xml.bind.DatatypeConverter");
+            } catch (ClassNotFoundException e) {
+                throw new GradleException("Cannot publish to S3 since the module 'java.xml.bind' is not available. "
+                    + "Please add \"-addmods java.xml.bind '-Dorg.gradle.jvmargs=-addmods java.xml.bind'\" to your GRADLE_OPTS.");
+            }
+        }
+    }
+
     private ClientConfiguration createConnectionProperties() {
         ClientConfiguration clientConfiguration = new ClientConfiguration();
         Optional<HttpProxySettings.HttpProxy> proxyOptional = s3ConnectionProperties.getProxy();
@@ -90,6 +107,7 @@ public class S3Client {
     }
 
     public void put(InputStream inputStream, Long contentLength, URI destination) {
+        checkRequiredJigsawModuleIsOnPath();
         try {
             S3RegionalResource s3RegionalResource = new S3RegionalResource(destination);
             String bucketName = s3RegionalResource.getBucketName();
@@ -104,7 +122,7 @@ public class S3Client {
 
             amazonS3Client.putObject(putObjectRequest);
         } catch (AmazonClientException e) {
-            throw ResourceException.putFailed(destination, e);
+            throw ResourceExceptions.putFailed(destination, e);
         }
     }
 
@@ -119,51 +137,26 @@ public class S3Client {
         return doGetS3Object(uri, false);
     }
 
-    public List<String> list(URI parent) {
-        List<String> results = new ArrayList<String>();
-
+    public List<String> listDirectChildren(URI parent) {
         S3RegionalResource s3RegionalResource = new S3RegionalResource(parent);
         String bucketName = s3RegionalResource.getBucketName();
         String s3BucketKey = s3RegionalResource.getKey();
         configureClient(s3RegionalResource);
 
         ListObjectsRequest listObjectsRequest = new ListObjectsRequest()
-                .withBucketName(bucketName)
-                .withPrefix(s3BucketKey)
-                .withMaxKeys(1000)
-                .withDelimiter("/");
+            .withBucketName(bucketName)
+            .withPrefix(s3BucketKey)
+            .withMaxKeys(1000)
+            .withDelimiter("/");
         ObjectListing objectListing = amazonS3Client.listObjects(listObjectsRequest);
-        results.addAll(resolveResourceNames(objectListing));
+        ImmutableList.Builder<String> builder = ImmutableList.builder();
+        builder.addAll(resourceResolver.resolveResourceNames(objectListing));
 
         while (objectListing.isTruncated()) {
             objectListing = amazonS3Client.listNextBatchOfObjects(objectListing);
-            results.addAll(resolveResourceNames(objectListing));
+            builder.addAll(resourceResolver.resolveResourceNames(objectListing));
         }
-        return results;
-    }
-
-    private List<String> resolveResourceNames(ObjectListing objectListing) {
-        List<String> results = new ArrayList<String>();
-        List<S3ObjectSummary> objectSummaries = objectListing.getObjectSummaries();
-        if (null != objectSummaries) {
-            for (S3ObjectSummary objectSummary : objectSummaries) {
-                String key = objectSummary.getKey();
-                String fileName = extractResourceName(key);
-                if (null != fileName) {
-                    results.add(fileName);
-                }
-            }
-        }
-        return results;
-    }
-
-    private String extractResourceName(String key) {
-        Matcher matcher = FILENAME_PATTERN.matcher(key);
-        if (matcher.find()) {
-            String group = matcher.group(0);
-            return group.contains(".") ? group : null;
-        }
-        return null;
+        return builder.build();
     }
 
     private S3Object doGetS3Object(URI uri, boolean isLightWeight) {
@@ -185,15 +178,19 @@ public class S3Client {
             if (null != errorCode && errorCode.equalsIgnoreCase("NoSuchKey")) {
                 return null;
             }
-            throw ResourceException.getFailed(uri, e);
+            throw ResourceExceptions.getFailed(uri, e);
         }
     }
 
     private void configureClient(S3RegionalResource s3RegionalResource) {
-        if (s3ConnectionProperties.getEndpoint().isPresent()) {
-            amazonS3Client.setEndpoint(s3ConnectionProperties.getEndpoint().get().toString());
+        Optional<URI> endpoint = s3ConnectionProperties.getEndpoint();
+        if (endpoint.isPresent()) {
+            amazonS3Client.setEndpoint(endpoint.get().toString());
         } else {
-            amazonS3Client.setRegion(s3RegionalResource.getRegion());
+            Optional<Region> region = s3RegionalResource.getRegion();
+            if (region.isPresent()) {
+                amazonS3Client.setRegion(region.get());
+            }
         }
     }
 }
